/*!
	\file Game.cpp

	\brief cpp-Datei der Klasse CGame
	\author Christian Roesch
	\date 21.6.2010

	Erklaerung	
*/

#include "Game.h"
#include "globals.h"

#include "GameObject.h"
#include "helper.h"
#include "resources.h"

#include "TextFileReader.h"
#include "StringHelper.h"

#include "Move.h"

#include <cmath>
#include <sstream>
#include <map>
#include <fstream>

#include <curl/curl.h>

std::string Game::m_strReceivedData;

/*!
	Der Konstruktor
*/
Game::Game()
:
m_pCurMove( new Move() )
{
	m_fGameTime= 0.0f;

	m_iMenuMode= MENU_PREGAME;
	m_strMenu[ MENU_PREGAME ].push_back( "Quit" );

	m_strMenu[ MENU_INGAME ].push_back( "Quit" );
	m_strMenu[ MENU_INGAME ].push_back( "Replay" );
	m_strMenu[ MENU_INGAME ].push_back( "Reset" );

	m_strMenu[ MENU_RETRY ].push_back( "Quit" );
	m_strMenu[ MENU_RETRY ].push_back( "Replay" );
	m_strMenu[ MENU_RETRY ].push_back( "Retry" );

	m_strMenu[ MENU_SLEEP ].push_back( "Quit" );
	m_strMenu[ MENU_SLEEP ].push_back( "Continue" );
	m_strMenu[ MENU_SLEEP ].push_back( "Replay" );

	m_strMenu[ MENU_NEXTLEVEL ].push_back( "Quit" );
	m_strMenu[ MENU_NEXTLEVEL ].push_back( "Replay" );
	m_strMenu[ MENU_NEXTLEVEL ].push_back( "Next Level" );

	m_strMenu[ MENU_EDITOR ].push_back( "Tiles" );
	m_strMenu[ MENU_EDITOR ].push_back( "Obj-Set" );
	m_strMenu[ MENU_EDITOR ].push_back( "Obj-Edit" );
	m_strMenu[ MENU_EDITOR ].push_back( "Turns" );
	m_strMenu[ MENU_EDITOR ].push_back( "Playtest" );

	m_strMenu[ MENU_OBJEDIT ].push_back( "Tiles" );
	m_strMenu[ MENU_OBJEDIT ].push_back( "Delete" );
	m_strMenu[ MENU_OBJEDIT ].push_back( "ToBack" );
	m_strMenu[ MENU_OBJEDIT ].push_back( "SetText" );
	
	m_strMenu[ MENU_TURNS ].push_back( "Tiles" );
	m_strMenu[ MENU_TURNS ].push_back( "Turn -" );
	m_strMenu[ MENU_TURNS ].push_back( "Turn +" );
	m_strMenu[ MENU_TURNS ].push_back( "Extra -" );
	m_strMenu[ MENU_TURNS ].push_back( "Extra +" );

	m_strMenu[ MENU_PLAYTEST ].push_back( "Sleep" );
	m_strMenu[ MENU_PLAYTEST ].push_back( "Previous" );
	m_strMenu[ MENU_PLAYTEST ].push_back( "Reset" );
	m_strMenu[ MENU_PLAYTEST ].push_back( "Edit" );

	m_bWantsToQuit= false;

	m_CurTextEdit= TEM_DISABLED;

	if( g_EditorEnabled )
	{
		m_EditObj.push_back( EditObjConfig( OT_MOVER_EAST , GameObject::MOVER, 0 ));
		m_EditObj.push_back( EditObjConfig( OT_MOVER_SOUTH , GameObject::MOVER, 1 ));
		m_EditObj.push_back( EditObjConfig( OT_MOVER_WEST , GameObject::MOVER, 2 ));
		m_EditObj.push_back( EditObjConfig( OT_MOVER_NORTH , GameObject::MOVER, 3 ));
		m_EditObj.push_back( EditObjConfig( OT_MOVER , GameObject::MOVER, -1 ));

		m_EditObj.push_back( EditObjConfig( OT_START , GameObject::GHOSTSTART, 0 ));
		m_EditObj.push_back( EditObjConfig( OT_TALKER , GameObject::TALKER, 0 ));

		m_EditObjID= -1;
		m_EditTile= -1;
	}
}

/*!
	Der Destruktor
*/
Game::~Game()
{
	delete m_pCurMove;
}

void Game::StartGame()
{
	m_GameState= GS_NORMAL;
	m_iMenuMode= MENU_INGAME;

	PrepareGameWorld( false );
}

void Game::ResetGame()
{
	m_Objects.resize( 0 );
	m_TileMap.resize( 0 );
	m_pCurMove->Reset();
	
	m_SelectionID= -1;
	m_HoverID= -1;
	m_PlayerID= -1;

	m_CurTextEdit= TEM_DISABLED;
}

void Game::PrepareGameWorld( bool ResetPlayer )
{
	ResetGame();

	if( g_EditorEnabled )
	{
		FrameWork::TextFileReader tfr;
		tfr.read(GetEditFileName());

		LoadLevel( tfr.getFileLines(), ResetPlayer );
		return;
	}

	int iLevelWanted= -1;
	if( m_bReplay )
	{
		iLevelWanted= m_Level;
	}
	if( GetLevelData( m_strNickName, m_strPassword, iLevelWanted ) )
	{
		std::vector<std::string> LevelData;
		std::string strLine;
		while( FrameWork::StringHelper::splitLoop( m_strReceivedData, "\n", strLine ) )
		{
			LevelData.push_back( strLine );
		}
		LoadLevel( LevelData, ResetPlayer );

		if( m_Level < g_iFirstSleepLevel || m_bReplay )
		{
			m_strMenu[ MENU_INGAME ][ 0 ]= "Quit";
		}
		else
		{
			m_strMenu[ MENU_INGAME ][ 0 ]= "Sleep";
		}


		if( m_strMessage != "" )
		{
			m_GameState= GS_COULDNOTLOAD;
			m_iMenuMode= MENU_RETRY;
		}
	}
	else
	{
		m_strMessage= "Could not get the current level from the web.";
		m_GameState= GS_COULDNOTLOAD;
		m_iMenuMode= MENU_RETRY;
	}
}

void Game::PrepareGame()
{
	m_Level= 0;
	m_bReplay= false;

	if( g_EditorEnabled )
	{
		char* pcCommand;
		pcCommand= GetCommandLine();
		{
			int i= 0;
			int iNumber= 0;
			int iBool= -1;
			{
				while( pcCommand[ i ] != 0 )
				{
					if(  pcCommand[ i ] == '+' )
					{
						iBool= 1;
						iNumber= 0;
					}
					else if(  pcCommand[ i ] == '-' )
					{
						iBool= 0;
						iNumber= 0;
					}
					else if( pcCommand[ i ] >= '0' && pcCommand[ i ] <= '9' )
					{
						iNumber= 10 * iNumber + pcCommand[ i  ] - '0';
					}
					else if(  pcCommand[ i ] == 'l' )
					{
						if( iNumber > 0 )
						{
							m_Level= iNumber;
						}
						iNumber= 0;
					}
					else
					{
						iNumber= 0;
						iBool= -1;
					}
					i++;
				}
			}
		}
	}

	ResetGame();

	m_GameState= GS_ENTERNICK;
	m_strPrompt= "Please enter your nickname:";
	m_strTextEdit= "";
	m_CurTextEdit= TEM_USER;
	m_iMenuMode= MENU_PREGAME;

	if( g_EditorEnabled )
	{
		m_GameState= GS_NORMAL;
		SelectTile();
	}
}

void Game::NextLevel()
{
	if( g_EditorEnabled )
	{
		m_iMenuMode= MENU_PLAYTEST;
		m_curEditMode= EM_PLAYTEST;

		m_ServerEmulator.erase( m_PlayTestUser );
		for(	std::map< int, GameObject >::iterator it = m_ServerEmulator.begin();
				it != m_ServerEmulator.end();
				++it )
		{
			it->second.m_iSubType+= m_iExtraTurns;
		}

		PrepareGameWorld( true );
		return;
	}

	m_iMenuMode= MENU_INGAME;
	PrepareGameWorld( !m_bReplay );
}


bool Game::FrameFunc()
{
	float dt=g_hge->Timer_GetDelta();
	m_fGameTime+= dt;

	// Input Mouse...
	MouseInput(dt);
	// Input keys
	KeyInput();

	FrameCalculation(dt);

	return m_bWantsToQuit;
}

void Game::TileToScreen( float& fX, float& fY )
{
	fX= (float)g_ScreenSizeX * 0.5f + ( ( fX * g_fTileSizeX - g_fCenterX ) * g_fZoom  );
	fY= (float)g_ScreenSizeY * 0.5f + ( ( fY * g_fTileSizeY - g_fCenterY ) * g_fZoom );
}

void Game::ScreenToTile( float& fX, float& fY )
{
	fX= ( fX + g_fCenterX * g_fZoom - (float)g_ScreenSizeX * 0.5f ) / ( g_fTileSizeX * g_fZoom );
	fY= ( fY + g_fCenterY * g_fZoom - (float)g_ScreenSizeY * 0.5f ) / ( g_fTileSizeY * g_fZoom );
}

bool Game::RenderFunc()
{
	// Render graphics
	g_hge->Gfx_BeginScene();
	g_hge->Gfx_Clear(0);

	DrawTiles();
	DrawObjects();

	if( g_EditorEnabled )
	{
		DrawEditor();

	}

	DrawUI();

	g_hge->Gfx_EndScene();
	return false;
}

int Game::GetLevel()
{
	return m_Level;
}

int Game::GetTile( int iX, int iY )
{
	if( iY < 0 || iY >= (int)m_TileMap.size() )
	{
		return c_BackgroundTile;
	}
	TileLine& tl= m_TileMap[ iY ];
	if( iX < 0 || iX >= (int)tl.m_TileType.size() )
	{
		return c_BackgroundTile;
	}
	return tl.m_TileType[ iX ];
}

void Game::AddAsciiLevelLine( std::string& strAct )
{
	TileLine line;
	for( size_t i= 0; i < strAct.size(); ++i )
	{
		int iTile= 0;
		switch( strAct[ i ] )
		{
		//oelig
		case '.':
			iTile= 4;
			break;
		case '_':
			iTile= 2;
			break;

		//normal/klebrig
		case 'b':
			iTile= 3;
			break;
		case 'd':
			iTile= 1;
			break;
		case 'X':
			iTile= 5;
			break;

		case '>':
			iTile= 16;
			break;
		case 'V':
			iTile= 17;
			break;
		case '<':
			iTile= 18;
			break;
		case 'A':
			iTile= 19;
			break;
		}

		line.m_TileType.push_back( iTile );
	}
	m_TileMap.push_back( line );
	m_iLevelSizeX= max(line.m_TileType.size(), m_iLevelSizeX );
}

void Game::AddBinLevelLine( std::string& strAct )
{
	TileLine line;
	for( size_t i= 0; i + 1 < strAct.size(); i+= 2 )
	{
		int iTile= (strAct[ i ] -'a') * 26 + (strAct[ i + 1 ] -'a');
		iTile= min( 255, iTile );
		iTile= max( 0, iTile );
		line.m_TileType.push_back( iTile );
	}
	m_TileMap.push_back( line );
	m_iLevelSizeX= max(line.m_TileType.size(), m_iLevelSizeX );
}

void Game::LoadLevel( std::vector <std::string>& LevelScript, bool bResetPlayer )
{
	m_strMessage= "";

	m_iTurns= 0;
	m_iLevelSizeX= 0;
	m_iLevelSizeY= 0;
	m_bReplay= false;
	bool bObject= false;
	GameObject goNext;

	int iCoopPlayer= 0;

	for( int i= 0; i < (int)LevelScript.size(); ++i )
	{
		std::string strAct= LevelScript[ i ];
		std::string strCommandFull;
		FrameWork::StringHelper::splitAt( strAct, "=", strCommandFull );
		FrameWork::StringHelper::toUpper( strCommandFull );

		std::stringstream ss;
		ss << strAct;

		if( strCommandFull == "LEVEL" )
		{
			FrameWork::StringHelper::trim( strAct, FrameWork::StringHelper::getSpaceTab() );
			AddAsciiLevelLine( strAct );
		}
		if( strCommandFull == "LBIN" )
		{
			FrameWork::StringHelper::trim( strAct, FrameWork::StringHelper::getSpaceTab() );
			AddBinLevelLine( strAct );
		}
		else if( strCommandFull == "TURNS" )
		{
			int iCurTruns= 0;
			ss >> iCurTruns;
			if( bResetPlayer )
			{
				m_iTurns= max( m_iTurns, iCurTruns );
			}
			else
			{
				m_iTurns= iCurTruns;
			}
			/*ss >> pInfo->m_v3Rot.x;
			ss >> pInfo->m_v3Rot.y;
			ss >> pInfo->m_v3Rot.z;*/
		}
		else if( strCommandFull == "EXTRATURNS" )
		{
			ss >> m_iExtraTurns;
		}
		else if( strCommandFull == "OBJECT" )
		{
			if( bObject )
			{
				m_Objects.push_back( goNext );
				goNext.Reset();
			}

			int Type= 0;
			ss >> Type;
			goNext.m_Type= (GameObject::ObjectType)Type;
			ss >> goNext.m_iSubType;
			bObject= true;

			if( goNext.m_Type == GameObject::OIL && !goNext.IsPlayer() )
			{
				iCoopPlayer++;
			}
		}
		else if( strCommandFull == "POS" )
		{
			ss >> goNext.m_iPosX;
			ss >> goNext.m_iPosY;
		}
		else if( strCommandFull == "POSXY" )
		{
			int XY= 0;
			ss >> XY;
			goNext.m_iPosX= ( XY ) & 0xffff;
			goNext.m_iPosY= ( XY >> 16 ) & 0xffff;
		}
		else if( strCommandFull == "TEXT" )
		{
			FrameWork::StringHelper::trim( strAct, FrameWork::StringHelper::getSpaceTab() );
			goNext.m_strText= strAct;
		}
		else if( strCommandFull == "LVL" )
		{
			ss >> m_Level;
		}
		else if( strCommandFull == "REPLAY" )
		{
			int i= 0;
			ss >> i;
			m_bReplay= (i == 1);
		}
		else if( strCommandFull == "MSG" )
		{
			FrameWork::StringHelper::trim( strAct, FrameWork::StringHelper::getSpaceTab() );
			m_strMessage= strAct;
		}
	}

	if( bObject )
	{
		m_Objects.push_back( goNext );
	}
	if( g_EditorEnabled )
	{
		if( m_curEditMode == EM_PLAYTEST )
		{
			for(	std::map< int, GameObject >::iterator it = m_ServerEmulator.begin();
				it != m_ServerEmulator.end();
				++it )
			{
				if( it->first != m_PlayTestUser )
				{
					m_Objects.push_back( it->second );
					m_Objects.back().m_iSubType= it->first;
					iCoopPlayer++;
				}
			}

			std::map< int, GameObject >::iterator it = m_ServerEmulator.find( m_PlayTestUser );
			if( it != m_ServerEmulator.end() && !bResetPlayer )
			{
				m_Objects.push_back( it->second );
				m_iTurns= m_Objects.back().m_iSubType;
				m_Objects.back().m_iSubType= -1;
				
				m_PlayerID= m_Objects.size() - 1;
			}
			else
			{
				AddPlayerObject( iCoopPlayer );
			}
		}
	}
	else
	{
		if( bResetPlayer )
		{
			RemovePlayer();
		}
		AddPlayerObject( iCoopPlayer );
	}

	g_fZoom= 1.0f;
	g_iZoom= 0;

	m_SelectionID= -1;
	m_HoverID= -1;

	m_iLevelSizeY= m_TileMap.size();
}

void Game::RemovePlayer()
{
	for( size_t i= 0; i < m_Objects.size(); ++i )
	{
		if( m_Objects[i].IsPlayer() )
		{
			m_Objects.erase( m_Objects.begin() + i );
			return;
		}
	}
}

void Game::SaveEditorLevel()
{
	// remove unecessary tiles
	for( size_t i= 0; i < m_TileMap.size(); ++i )
	{
		TileLine& l= m_TileMap[ i ];
		while( l.m_TileType.size() > 0 && l.m_TileType.back() == 0 )
		{
			l.m_TileType.pop_back();
		}
	}
	while( m_TileMap.size() > 0 && m_TileMap.back().m_TileType.size() == 0 )
	{
		m_TileMap.pop_back();
	}

	std::ofstream ofs( GetEditFileName().c_str() );

	ofs << "TURNS=" << m_iTurns << std::endl;
	ofs << "EXTRATURNS=" << m_iExtraTurns << std::endl;
	ofs << std::endl;

	for( size_t i= 0; i < m_TileMap.size(); ++i )
	{
		ofs << "LBIN=";
		TileLine& l= m_TileMap[ i ];
		for( size_t j= 0; j < l.m_TileType.size(); ++j )
		{
			int iTile= l.m_TileType[ j ];
			iTile= min( 255, iTile );
			iTile= max( 0, iTile );
			char c1= ('a' + ( iTile / 26 ) % 26 );
			char c2= ('a' + ( iTile / 1 ) % 26 );
			ofs << c1 << c2;
		}
		ofs << std::endl;
	}
	ofs << std::endl;

	for( size_t i= 0; i < m_Objects.size(); ++i )
	{
		GameObject& o= m_Objects[ i ];

		if( o.m_Type == GameObject::OIL )
		{
			continue;
		}

		ofs << "OBJECT=" <<	o.m_Type << " " << o.m_iSubType << std::endl;
		ofs << "POS=" << o.m_iPosX << " " << o.m_iPosY << std::endl;
		ofs << "TEXT=" << o.m_strText << std::endl;
	}
}

void Game::AddPlayerObject( int iStartPosOffset )
{
	for( size_t i= 0; i < m_Objects.size(); ++i )
	{
		if( m_Objects[i].IsPlayer() )
		{
			m_PlayerID= i;
			return;
		}
	}

	GameObject go;
	go.m_iPosX= 0;
	go.m_iPosY= 0;
	go.m_Type= GameObject::OIL;
	go.m_iSubType= -1;
	if( m_Level == 0 )
	{
		go.m_strText= "You can select me and move me around with the cursor keys.";
	}

	std::map<int,int> Starts;
	for( size_t i= 0; i < m_Objects.size(); ++i )
	{
		GameObject& o= m_Objects[ i ];
		if( o.IsStart() )
		{
			Starts[ Starts.size() ]= i;
		}
	}
	if( Starts.size() > 0 )
	{
		iStartPosOffset= iStartPosOffset % Starts.size();
		GameObject& o= m_Objects[ Starts[ iStartPosOffset ] ];
		go.m_iPosX= o.m_iPosX;
		go.m_iPosY= o.m_iPosY;
	}

	g_fCenterX= ((float)go.m_iPosX + 0.5f ) * g_fTileSizeX;
	g_fCenterY= ((float)go.m_iPosY + 0.5f ) * g_fTileSizeY;

	m_Objects.push_back(go);
	m_PlayerID= m_Objects.size() - 1;
}

void Game::ExecuteMove()
{
	int ObjectID= m_pCurMove->m_iObjectID;
	GameObject& o= m_Objects[ ObjectID ];
	int iTargetX;
	int iTargetY;
	m_pCurMove->GetTarget( iTargetX, iTargetY );
	o.m_iPosX= iTargetX;
	o.m_iPosY= iTargetY;
	o.m_pMove= NULL;

	if( IsSliding( o.m_iPosX, o.m_iPosY ) )
	{
		m_pCurMove->m_fTimeLeft= 1.0f;
		m_pCurMove->m_iPosX= o.m_iPosX;
		m_pCurMove->m_iPosY= o.m_iPosY;
		int iNewDir= IsChangingDir( o.m_iPosX, o.m_iPosY );
		if( iNewDir != -1 )
		{
			m_pCurMove->m_iDir= iNewDir;
		}
		MoveAllowed( m_pCurMove );
	}
	else
	{
		m_pCurMove->m_iObjectID= -1;
	}

	if( m_pCurMove->m_iObjectID == -1)
	{
		CheckAction( ObjectID );
	}
}

bool Game::MoveAllowed( Move* pMove )
{
	int iTargetX;
	int iTargetY;
	pMove->GetTarget( iTargetX, iTargetY );
	if( !IsWalkable( iTargetX, iTargetY ) )
	{
		pMove->m_iObjectID= -1;
		return false;
	}

	GameObject& o= m_Objects[ pMove->m_iObjectID ];
	if( !o.AcceptsMove( pMove ) )
	{
		pMove->m_iObjectID= -1;
		return false;
	}
	return true;
}

bool Game::IsWalkable( int iX, int iY )
{
	int iTile= GetTile( iX, iY ) % 32;
	switch( iTile )
	{
	//list of blocking Tiles
	case 0:
		return false;
	}

	for( size_t i= 0; i < m_Objects.size(); ++i )
	{
		GameObject& o= m_Objects[ i ];
		if( !o.IsBlocking() )
		{
			continue;
		}
		if( iX == o.m_iPosX && iY == o.m_iPosY )
		{
			return false;
		}
	}

	return true;
}

bool Game::IsSliding( int iX, int iY )
{
	int iTile= GetTile( iX, iY ) % 32;
	switch( iTile )
	{
	//list of stopping Tiles
	case 0:
	case 1:
	case 3:
	case 6:
		return false;
	}

	return true;
}

int Game::IsWin( int iX, int iY )
{
	int iTile= GetTile( iX, iY ) % 32;
	switch( iTile )
	{
	//list off winning Tiles
	case 5:
	case 6:
		return true;
	}

	return false;
}

int Game::IsChangingDir( int iX, int iY )
{
	int iTile= GetTile( iX, iY ) % 32;
	if( iTile >= 16 && iTile <= 19 ) 
	{
		return iTile - 16;
	}
	return -1;
}

bool Game::IsTurnActive()
{
	if( m_pCurMove->m_iObjectID != -1 )
	{
		return true;
	}
	return false;
}

void Game::DrawTiles()
{
	int iTopX= -1 + (int)floor( ( g_fCenterX * g_fZoom - (float)g_ScreenSizeX * 0.5f ) / ( g_fTileSizeX * g_fZoom ) );
	int iTopY= -1 + (int)floor( ( g_fCenterY * g_fZoom - (float)g_ScreenSizeY * 0.5f ) / ( g_fTileSizeY * g_fZoom ));
	int iEndX= 1 + (int)ceil( ( g_fCenterX * g_fZoom + (float)g_ScreenSizeX * 0.5f ) / ( g_fTileSizeX * g_fZoom ) );
	int iEndY= 1 + (int)ceil( ( g_fCenterY * g_fZoom + (float)g_ScreenSizeY * 0.5f ) / ( g_fTileSizeY * g_fZoom ) );

	for( int iY= iTopY; iY <= iEndY; ++iY )
	{
		for( int iX= iTopX; iX <= iEndX; ++iX )
		{
			if( g_EditorEnabled )
			{
				sprTile[ 0 ]->SetColor( ( iX < 0 || iY < 0 ) ? 0xff808080 : 0xffffffff );
			}
			int iTile= GetTile( iX, iY );
			float fX= (float)iX;
			float fY= (float)iY;
			TileToScreen( fX, fY );
			sprTile[ iTile ]->RenderEx(
				fX,
				fY,
				0.0f,
				g_fZoom );
		}
	}
}

void Game::DrawObjects()
{
	for( size_t i= 0; i < m_Objects.size(); ++i )
	{
		m_Objects[ i ].draw();

		GameObject& o= m_Objects[ i ];
		if( !IsTurnActive() )
		{
			if( i == m_HoverID || i == m_SelectionID )
			{
				ObjectTextures ot= OT_SELECT;
				float fPosX= (float)o.m_iPosX;
				float fPosY= (float)o.m_iPosY;
				g_Game.TileToScreen( fPosX, fPosY );
				float fZoom= g_fZoom;
				//if( i == m_SelectionID )
				sprObj[ ot ]->RenderEx( fPosX, fPosY, 0.0f, fZoom );
			}
		}

		if( g_EditorEnabled )
		{
			float fPosX= (float)o.m_iPosX;
			float fPosY= (float)o.m_iPosY;
			g_Game.TileToScreen( fPosX, fPosY );
			if( m_curEditMode == EM_OBJECTEDIT )
			{
				if( o.IsStart() )
				{
					fnt->SetColor( 0xc000ff00 );
					fnt->printfb( 
						fPosX-24, 
						fPosY-24,
						48,
						48,
						HGETEXT_CENTER|HGETEXT_MIDDLE,
						"%d", 
						i + 1 );
				}

				if( i == m_EditObjID )
				{
					ObjectTextures ot= OT_SELECT;
					sprObj[ ot ]->RenderEx( fPosX, fPosY, 0.0f, g_fZoom );
				}
			}

		}
	}
}

void Game::DrawUI()
{
	if( m_HoverID != -1 && !IsTurnActive() )
	{
		GameObject& o= m_Objects[ m_HoverID ];
		float fX= (float)o.m_iPosX;
		float fY= (float)o.m_iPosY;
		TileToScreen( fX, fY );
		fY-= g_fTileSizeY;
		float fWidth= max( 200.0f, g_ScreenSizeX - fX - 16.0f );
		fWidth= min( 500.0f, fWidth );
		float fHeight= 500.0f;
		fnt->SetColor(0xff000000);
		fnt->printfb(fX , fY - fHeight, fWidth, fHeight, HGETEXT_BOTTOM|HGETEXT_LEFT, o.m_strText.c_str() );
		fnt->SetColor(0xffffffff);
		fX-= 2.0f;
		fY-= 2.0f;
		fnt->printfb(fX , fY - fHeight, fWidth, fHeight, HGETEXT_BOTTOM|HGETEXT_LEFT, o.m_strText.c_str() );
	}


	fnt->SetColor(0xffffffff);
	sprTextBack->Render(
		g_ScreenSizeX - 128 - 2, 
		2 );
	fnt->printfb( 
		g_ScreenSizeX - 128 - 2, 
		2,
		128,
		32,
		HGETEXT_CENTER|HGETEXT_MIDDLE,
		"Level %02d", 
		GetLevel() + 1 );

	sprTextBack->Render(
		g_ScreenSizeX - 260 - 2, 
		2 );
	fnt->printfb( 
		g_ScreenSizeX - 260 - 2, 
		2,
		128,
		32,
		HGETEXT_CENTER|HGETEXT_MIDDLE,
		"Turn %02d", 
		m_iTurns );

	int iMenu= OnMenu();
	for( int i= 0; i < m_strMenu[ m_iMenuMode ].size(); ++i )
	{
		fnt->SetColor( iMenu == i ? 0xffa0a0ff : 0xff4080e0);	
		float fX= 2 + 132 * i;
		float fY= 2;
		sprTextBack->Render(
			fX, 
			fY );
		fnt->printfb( 
			fX, 
			fY,
			128,
			32,
			HGETEXT_CENTER|HGETEXT_MIDDLE,
			m_strMenu[ m_iMenuMode ][ i ].c_str() );
	}

	float fMouseX;
	float fMouseY;
	g_hge->Input_GetMousePos(&fMouseX, &fMouseY);

	if( m_CurTextEdit != TEM_DISABLED )
	{
		sprEditBack->Render(
		g_ScreenSizeX / 2 - 256,
		g_ScreenSizeY / 2 - 256 );


		fnt->SetColor(0xffffffff);
		fnt->printfb( 
			g_ScreenSizeX / 2 - 250,
			g_ScreenSizeY / 2 - 256,
			500,
			96,
			HGETEXT_CENTER|HGETEXT_MIDDLE,
			m_strPrompt.c_str() );


		std::string strOut= m_strTextEdit;
		if( m_CurTextEdit == TEM_PASS )
		{
			strOut="";
			while( strOut.size() < m_strTextEdit.size() )
			{
				strOut+="*";
			}
		}
		if( ((int)( m_fGameTime * 5.0f ) & 1 ) == 0 )
		{
			strOut+= "|";
		}
		fnt->printfb( 
			g_ScreenSizeX / 2 - 250,
			g_ScreenSizeY / 2 - 128,
			500,
			384,
			HGETEXT_CENTER|HGETEXT_MIDDLE,
			strOut.c_str() );
	}

	if( m_strMessage != "" )
	{
		sprMsgBack->Render(
		g_ScreenSizeX / 2 - 256,
		g_ScreenSizeY / 2 - 256 );

		fnt->SetColor(0xffffffff);
		fnt->printfb( 
			g_ScreenSizeX / 2 - 250,
			g_ScreenSizeY / 2 - 250,
			500,
			500,
			HGETEXT_CENTER|HGETEXT_MIDDLE,
			m_strMessage.c_str() );
	}

	sprCursor->RenderEx(
		fMouseX,
		fMouseY,
		0.0f,
		g_fZoom );
}

int Game::OnMenu()
{
	float fMouseX;
	float fMouseY;
	g_hge->Input_GetMousePos(&fMouseX, &fMouseY);
	for( int i= 0; i < m_strMenu[ m_iMenuMode ].size(); ++i )
	{
		float fX= 2 + 132 * i;
		float fY= 2;
		if( fMouseX > fX && fMouseX < fX + 128.0f &&
			fMouseY > fY && fMouseY < fY + 32.0f )
		{
			return i;
		}
	}
	return -1;
}

void Game::MouseInput( float dt )
{
	int Wheel= g_hge->Input_GetMouseWheel();
	if( Wheel > 0 )
	{
		g_iZoom--;
	}
	if( Wheel < 0 )
	{
		g_iZoom++;
	}
	g_iZoom= max( -6, g_iZoom );
	g_iZoom= min( 6, g_iZoom );
	g_fZoom= pow( 1.0f / 2.0f, 0.25f * (float)g_iZoom );

	float fMouseX;
	float fMouseY;
	g_hge->Input_GetMousePos(&fMouseX, &fMouseY); 

	int iMenu= OnMenu();

	if( iMenu != -1 )
	{
		if( g_hge->Input_KeyDown(HGEK_LBUTTON) )
		{
			OnMouseClick(iMenu);
		}
	}
	else
	{
		float fSpeed= 512.0f * dt;
		if( fMouseX < 16.0f )
		{
			g_fCenterX-= fSpeed;
			if( g_fCenterX < 0.0f )
			{
				g_fCenterX= 0.0f;
			}
		}
		if( fMouseX > g_ScreenSizeX - 16.0f )
		{
			g_fCenterX+= fSpeed;
			if( g_fCenterX > m_iLevelSizeX * g_fTileSizeX )
			{
				g_fCenterX= m_iLevelSizeX * g_fTileSizeX;
			}
		}
		if( fMouseY < 16.0f )
		{
			g_fCenterY-= fSpeed;
			if( g_fCenterY < 0.0f )
			{
				g_fCenterY= 0.0f;
			}
		}
		if( fMouseY > g_ScreenSizeY - 16.0f )
		{
			g_fCenterY+= fSpeed;
			if( g_fCenterY > m_iLevelSizeY * g_fTileSizeY )
			{
				g_fCenterY= m_iLevelSizeY * g_fTileSizeY;
			}
		}

		if( IsIngameActive() )
		{
			float fMouseTileX= fMouseX;
			float fMouseTileY= fMouseY;
			ScreenToTile( fMouseTileX, fMouseTileY );
			int iTileX= (int)floor( fMouseTileX + 0.5f );
			int iTileY= (int)floor( fMouseTileY + 0.5f );
			m_HoverID= -1;
			for( size_t i= 0; i < m_Objects.size(); ++i )
			{
				GameObject& o= m_Objects[ i ];
				if( o.m_iPosX == iTileX && o.m_iPosY == iTileY )
				{
					if( o.IsSelectable() )
					{
						m_HoverID= i;
					}
				}
			}

			if (g_hge->Input_GetKeyState(HGEK_LBUTTON) )
			{
				m_SelectionID= m_HoverID;
			}

			if (g_hge->Input_KeyDown(HGEK_RBUTTON))
			{
				if( m_HoverID > -1 )
				{
					m_Objects[ m_HoverID ].TryToDisable();
					if( m_SelectionID == m_HoverID )
					{
						m_SelectionID= -1;
					};
				}
			}
		}
		else if( g_EditorEnabled )
		{
			bool bLeft= g_hge->Input_KeyDown(HGEK_LBUTTON);
			bool bRight= g_hge->Input_GetKeyState(HGEK_RBUTTON);
			switch( m_curEditMode )
			{
			case EM_SELECTTILES:
				if( bLeft )
				{
					int iX= (int)((fMouseX - 56.0f )/ 32.0f);
					int iY= (int)((fMouseY - 56.0f )/ 32.0f);
					m_EditTile= iX + iY * 16;
					m_EditTile= min( m_EditTile, 255 );
					m_EditTile= max( m_EditTile, 0 );
					m_curEditMode= EM_TILES;
				}
				break;
			case EM_TILES:
				if( bLeft || bRight )
				{
					float fMouseTileX= fMouseX;
					float fMouseTileY= fMouseY;
					ScreenToTile( fMouseTileX, fMouseTileY );
					int iTileX= (int)floor( fMouseTileX + 0.5f );
					int iTileY= (int)floor( fMouseTileY + 0.5f );
					SetTile( iTileX, iTileY, m_EditTile );
				}
				break;
			case EM_SELECTOBJECT:
				if( bLeft )
				{
					int iX= (int)((fMouseX - 56.0f )/ 32.0f);
					int iY= (int)((fMouseY - 56.0f )/ 32.0f);
					m_EditObjID= iX + iY * 5;
					m_EditObjID= min( m_EditObjID, m_EditObj.size() - 1 );
					m_EditObjID= max( m_EditObjID, 0 );
					m_curEditMode= EM_OBJECTS;
				}
				break;
			case EM_OBJECTS:
				if( bLeft )
				{
					if( m_EditObjID >= 0)
					{
						float fMouseTileX= fMouseX;
						float fMouseTileY= fMouseY;
						ScreenToTile( fMouseTileX, fMouseTileY );
						int iTileX= (int)floor( fMouseTileX + 0.5f );
						int iTileY= (int)floor( fMouseTileY + 0.5f );

						GameObject o;
						o.m_iPosX= iTileX;
						o.m_iPosY= iTileY;
						o.m_iSubType= m_EditObj[ m_EditObjID ].m_iSubType;
						o.m_Type= (GameObject::ObjectType)m_EditObj[ m_EditObjID ].m_iType;
						m_Objects.push_back( o );
					}
				}
				break;
			case EM_OBJECTEDIT:
				if( bLeft )
				{
					float fMouseTileX= fMouseX;
					float fMouseTileY= fMouseY;
					ScreenToTile( fMouseTileX, fMouseTileY );
					int iTileX= (int)floor( fMouseTileX + 0.5f );
					int iTileY= (int)floor( fMouseTileY + 0.5f );
					m_EditObjID= -1;
					for( size_t i= 0; i < m_Objects.size(); ++i )
					{
						GameObject& o= m_Objects[ i ];
						if( o.m_iPosX == iTileX && o.m_iPosY == iTileY )
						{
							m_EditObjID= i;
						}
					}
				}
				break;
			}
		}
	}
}

void Game::OnMouseClick( int iMenu )
{
	switch(m_iMenuMode)
	{
	case MENU_PREGAME:
		switch( iMenu )
		{
		case 0:
			m_bWantsToQuit= true;
			break;
		}
		break;
	case MENU_INGAME:
		switch( iMenu )
		{
		case 0:
			if( m_Level < g_iFirstSleepLevel || m_bReplay )
			{
				m_bWantsToQuit= true;
			}
			else
			{
				m_strPrompt= "Please enter your sleeping message:";
				m_strTextEdit= "I am tired!";
				m_CurTextEdit= TEM_SLEEPMSG;
			}
			break;
		case 1:
			SelectLevel();
			break;
		case 2:
			PrepareGameWorld( true );
			break;
		}
		break;
	case MENU_NEXTLEVEL:
		switch( iMenu )
		{
		case 0:
			m_bWantsToQuit= true;
			break;
		case 1:
			m_strMessage= "";
			SelectLevel();
			break;
		case 2:
			m_strMessage= "";
			NextLevel();
			break;
		}
		break;
	case MENU_RETRY:
		switch( iMenu )
		{
		case 0:
			m_bWantsToQuit= true;
			break;
		case 1:
			m_strMessage= "";
			SelectLevel();
			break;
		case 2:
			m_strMessage= "";
			if( m_GameState == GS_COULDNOTLOAD )
			{
				m_iMenuMode= MENU_INGAME;
				PrepareGameWorld( false );
			}
			else if( m_GameState == GS_COULDNOTSLEEP )
			{
				SleepLevel();
			}
			else if( m_GameState == GS_COULDNOTSOLVE )
			{
				WinLevel();
			}
			break;
		}
		break;
	case MENU_SLEEP:
		switch( iMenu )
		{
		case 0:
			m_bWantsToQuit= true;
			break;
		case 1:
			m_strMessage= "";
			m_iMenuMode= MENU_INGAME;
			break;
		case 2:
			m_strMessage= "";
			SelectLevel();
			break;
		}
		break;
	case MENU_EDITOR:
		SaveEditorLevel();
		switch( iMenu )
		{
		case 0:
			SelectTile();
			break;
		case 1:
			SelectObject();
			break;
		case 2:
			EditObject();
			break;
		case 3:
			EditTurns();
			break;
		case 4:
			PlayTest();
			break;
		}
		break;
	case MENU_OBJEDIT:
		SaveEditorLevel();
		switch( iMenu )
		{
		case 0:
			SelectTile();
			break;
		case 1: //Delete
			if( m_EditObjID >= 0 && m_EditObjID < m_Objects.size() )
			{
				m_Objects.erase( m_Objects.begin() + m_EditObjID );
				m_EditObjID= -1;
			}
			break;
		case 2: //ToBack
			if( m_EditObjID >= 0 && m_EditObjID < m_Objects.size() )
			{
				GameObject o= m_Objects[ m_EditObjID ];
				m_Objects.erase( m_Objects.begin() + m_EditObjID );
				m_Objects.insert( m_Objects.begin(), o );
				m_EditObjID= 0;
			}
			break;
		case 3: //SetText
			if( m_EditObjID >= 0 && m_EditObjID < m_Objects.size() )
			{
				m_strPrompt= "Enter Object Text:";
				m_strTextEdit= m_Objects[ m_EditObjID ].m_strText;
				m_CurTextEdit= TEM_EDITOROBJTEXT;
			}
			break;
		}
		break;
	case MENU_TURNS:
		SaveEditorLevel();
		switch( iMenu )
		{
		case 0:
			SelectTile();
			break;
		case 1:
			m_iTurns--;
			break;
		case 2:
			m_iTurns++;
			break;
		case 3:
			m_iExtraTurns--;
			break;
		case 4:
			m_iExtraTurns++;
			break;
		}
		break;
	case MENU_PLAYTEST:
		switch( iMenu )
		{
		case 0:
			if( g_EditorEnabled )
			{
				int UserDir= 1;
				SleepPlayTest(UserDir);
			}
			break;
		case 1:
			if( g_EditorEnabled )
			{
				int UserDir= -1;
				SleepPlayTest(UserDir);
			}
			break;
		case 2:
			PrepareGameWorld( true );
			break;
		case 3:
			SelectTile();
			break;
		}
		break;
	}
}

void Game::OnTextEnter( bool bOk )
{
	switch( m_CurTextEdit )
	{
	case TEM_USER:
		if( !bOk )
		{
			m_bWantsToQuit= true;
		}
		else
		{
			m_strNickName= m_strTextEdit;
			bool bUserExists= FindUser( m_strNickName );
			if( bUserExists )
			{
				m_GameState= GS_ENTERPASS;
				m_strPrompt= "Please enter your password:";
				m_strTextEdit= "";
				m_CurTextEdit= TEM_PASS;
				return;
			}
			else
			{
				m_GameState= GS_CREATEACCOUNT;
				m_strPrompt= "Account does not exist.\nEnter a password to create account \"";
				m_strPrompt+= m_strNickName;
				m_strPrompt+= "\":";
				m_strTextEdit= "";
				m_CurTextEdit= TEM_PASS;
				return;
			}
		}
		break;
	case TEM_PASS:
		if( !bOk )
		{
			m_bWantsToQuit= true;
		}
		else
		{
			m_strPassword= m_strTextEdit;
			if( m_GameState == GS_ENTERPASS )
			{
				bool bLogin= Login( m_strNickName, m_strPassword );
				if( bLogin  ) // login ok
				{
					StartGame();
				}
				else // password does not match
				{
					m_GameState= GS_ENTERNICK;
					m_strPrompt= "Login failed. Try again!\nPlease enter your nickname:";
					m_strTextEdit= "";
					m_CurTextEdit= TEM_USER;
					return;
				}
			}
			else if( m_GameState == GS_CREATEACCOUNT )
			{
				bool bCreated= CreateUser( m_strNickName, m_strPassword );
				if( bCreated  )
				{
					StartGame();
				}
				else // password does not match
				{
					m_GameState= GS_ENTERNICK;
					m_strPrompt= "Account cannot be created. Try again!\nPlease enter your nickname:";
					m_strTextEdit= "";
					m_CurTextEdit= TEM_USER;
					return;
				}
			}
		}
		break;
	case TEM_SLEEPMSG:
		{
			if( bOk )
			{
				SleepLevel();
			}
		}
		break;
	case TEM_LEVEL:
		{
			if( bOk )
			{
				m_bReplay= true;
				std::stringstream ss;
				ss << m_strTextEdit;
				ss >> m_Level;
				m_Level-= 1;

				StartGame();
			}
		}
		break;
	case TEM_EDITOROBJTEXT:
		if( g_EditorEnabled )
		{
			if( m_EditObjID >= 0 && m_EditObjID < m_Objects.size() )
			{
				if( bOk )
				{
					m_Objects[ m_EditObjID ].m_strText= m_strTextEdit;
				}
			}
		}
		break;
	}

	m_CurTextEdit= TEM_DISABLED;
}

void Game::KeyInput()
{
	if( m_CurTextEdit != TEM_DISABLED )
	{
		int iKey= g_hge->Input_GetKey();
		if ( iKey == HGEK_ENTER )
		{
			OnTextEnter( true );
		}
		else if ( iKey == HGEK_ESCAPE )
		{
			OnTextEnter( false );
		}
		else if ( iKey == HGEK_DELETE || iKey == HGEK_BACKSPACE )
		{
			if( m_strTextEdit.size() > 0 )
			{
				m_strTextEdit= m_strTextEdit.substr( 0, m_strTextEdit.size() - 1 );
			}
		}
		else
		{
			int iMax= 30;
			if( m_CurTextEdit == TEM_EDITOROBJTEXT || 
				m_CurTextEdit == TEM_SLEEPMSG )
			{
				int iMax= 160;
			}
			if( m_CurTextEdit == TEM_LEVEL )
			{
				int iMax= 4;
			}
			if( m_strTextEdit.size() < iMax )
			{
				int c= g_hge->Input_GetChar();
				if( c >= 32 && c <= 126 )
				{
					switch( c )
					{
					case '<':
					case '>':
					case ';':
						break;
					default:
						m_strTextEdit+= c;
					}
				}
			}
		}
		return;
	}


	if( g_EditorEnabled )
	{
		if (g_hge->Input_KeyDown(HGEK_ESCAPE))
		{
			m_bWantsToQuit= true;
		}
	}

	bool bDirs[ 4 ];
	bDirs[ DIR_NORTH ]= g_hge->Input_KeyDown(HGEK_UP);
	bDirs[ DIR_WEST ]= g_hge->Input_KeyDown(HGEK_LEFT);
	bDirs[ DIR_EAST ]= g_hge->Input_KeyDown(HGEK_RIGHT);
	bDirs[ DIR_SOUTH ]= g_hge->Input_KeyDown(HGEK_DOWN);
	if( !IsTurnActive() && m_iTurns > 0 && IsIngameActive() )
	{
		for( int i= 0; i < 4; ++i )
		{
			if( bDirs[ i ] && m_SelectionID != -1 && m_pCurMove->m_iObjectID == -1 )
			{
				GameObject& o= m_Objects[ m_SelectionID ];
				m_pCurMove->m_iObjectID= m_SelectionID;
				m_pCurMove->m_iDir= i;
				m_pCurMove->m_fTimeLeft= 1.0f;
				m_pCurMove->m_iPosX= o.m_iPosX;
				m_pCurMove->m_iPosY= o.m_iPosY;
				if( MoveAllowed( m_pCurMove ) )
				{
					m_iTurns--;
				}
			}
		}
	}

	if( g_EditorEnabled )
	{
		if( m_curEditMode == EM_PLAYTEST )
		{
			if (g_hge->Input_KeyDown(HGEK_PGDN) )
			{
				SleepPlayTest(-1);
			}
			if (g_hge->Input_KeyDown(HGEK_PGUP) )
			{
				SleepPlayTest(1);
			}
		}

		if( m_curEditMode == EM_OBJECTEDIT )
		{
			for( int i= 0; i < 4; ++i )
			{
				if( bDirs[ i ] )
				{
					if( m_EditObjID >= 0 && m_EditObjID < m_Objects.size() )
					{
						GameObject& o= m_Objects[ m_EditObjID ];
						int dX;
						int dY;
						DirToVec( i, dX, dY );
						o.m_iPosX+= dX;
						o.m_iPosY+= dY;
					}
				}
			}
		}
		if( m_curEditMode == EM_TILES || m_curEditMode == EM_OBJECTS )
		{
			if( g_hge->Input_GetKeyState(HGEK_CTRL) )
			{
				for( int i= 0; i < 4; ++i )
				{
					if( bDirs[ i ] )
					{
						int dX;
						int dY;
						DirToVec( i, dX, dY );
						EditMoveLevel( dX, dY );
					}
				}
			}
		}
		if( m_curEditMode == EM_TILES || m_curEditMode == EM_OBJECTS || m_curEditMode == EM_OBJECTEDIT )
		{
			if( g_hge->Input_GetKeyState(HGEK_CTRL) )
			{
				if( g_hge->Input_KeyDown(HGEK_L) )
				{
					m_EditObjID= 0;

					PrepareGameWorld( true );
				}
			}
		}
	}
}

void Game::FrameCalculation( float dt )
{
	//do frame calculation
	if( m_pCurMove->m_iObjectID != -1 )
	{
		m_pCurMove->m_fTimeLeft-= dt * g_fGameSpeed;
		if( m_pCurMove->m_fTimeLeft < 0.0f )
		{
			ExecuteMove();
		}
	}
}

void Game::CheckAction( int ObjectID )
{
	GameObject& o= m_Objects[ ObjectID ];
	if( o.IsPlayer() )
	{
		if( IsWin( o.m_iPosX, o.m_iPosY ) )
		{
			WinLevel();
		}
	}

	if( o.IsOil() )
	{
		//Leak();
	}
}

void Game::WinLevel()
{
	if( g_EditorEnabled )
	{
		m_strMessage= "Level solved!\nContinue to next level.";
		m_iMenuMode= MENU_NEXTLEVEL;
		return;
	}

	if( m_bReplay )
	{
		m_strMessage= "You can replay the next level now.";
		m_iMenuMode= MENU_NEXTLEVEL;
		m_Level++;
		return;
	}

	if( Solve( m_strNickName, m_strPassword, m_Level ) )
	{
		m_strMessage= "Level solved!\nContinue to next level.";
		m_iMenuMode= MENU_NEXTLEVEL;
	}
	else
	{
		m_strMessage= "Could not connect to the web and save your progress.";
		m_GameState= GS_COULDNOTSOLVE;
		m_iMenuMode= MENU_RETRY;
	}
}

void Game::SleepLevel()
{
	if( Sleep( m_strNickName, m_strPassword ) )
	{
		m_strMessage= "Your progress is stored and you can go to sleep.";
		m_iMenuMode= MENU_SLEEP;
	}
	else
	{
		m_strMessage= "Could not connect to the web and save your progress.";
		m_GameState= GS_COULDNOTSLEEP;
		m_iMenuMode= MENU_RETRY;
	}
}


void Game::PlayTest()
{
	m_ServerEmulator.clear();
	m_curEditMode= EM_PLAYTEST;
	m_iMenuMode= MENU_PLAYTEST;
	m_PlayTestUser= 0;

	PrepareGameWorld( true );
}

void Game::SelectTile()
{
	m_curEditMode= EM_SELECTTILES;
	m_iMenuMode= MENU_EDITOR;

	PrepareGameWorld( true );
}

void Game::SelectObject()
{
	m_EditObjID= 0;

	m_curEditMode= EM_SELECTOBJECT;
	m_iMenuMode= MENU_EDITOR;

	PrepareGameWorld( true );
}

void Game::EditObject()
{
	m_EditObjID= 0;

	m_curEditMode= EM_OBJECTEDIT;
	m_iMenuMode= MENU_OBJEDIT;

	PrepareGameWorld( true );
}

void Game::EditTurns()
{
	m_curEditMode= EM_TURNS;
	m_iMenuMode= MENU_TURNS;
	PrepareGameWorld( true );
}

std::string Game::GetEditFileName()
{
	char pcName[]= "dev/000level.txt";
	pcName[ 4 ]= '0' + (m_Level / 100 ) % 10;
	pcName[ 5 ]= '0' + (m_Level / 10 ) % 10;
	pcName[ 6 ]= '0' + (m_Level / 1 ) % 10;

	return std::string( pcName );
}

bool Game::IsIngameActive()
{
	if( g_EditorEnabled )
	{
		if( m_iMenuMode == MENU_PLAYTEST )
		{
			return true;
		}
	}
	return m_iMenuMode == MENU_INGAME;
}

void Game::SleepPlayTest( int UserDir )
{
	GameObject o= m_Objects[ m_PlayerID ];
	o.m_pMove= NULL;
	{
		std::stringstream ss;
		ss << "Playtest User " << ( m_PlayTestUser + 1 );
		o.m_strText= ss.str();
	}
	o.m_iSubType= m_iTurns;
	m_ServerEmulator[ m_PlayTestUser ]= o;
	m_PlayTestUser+= UserDir;
	if( m_PlayTestUser < 0 )
	{
		m_PlayTestUser= 0;
	}

	PrepareGameWorld( false );
}

void Game::DrawEditor()
{
	if( m_curEditMode == EM_PLAYTEST )
	{
		fnt->SetColor(0xffffffff);
		fnt->printfb( 
			g_ScreenSizeX - 128 - 2, 
			34,
			128,
			32,
			HGETEXT_CENTER|HGETEXT_MIDDLE,
			"Player %02d", 
			m_PlayTestUser + 1 );
	}
	else
	{
		fnt->SetColor(0xffffffff);
		/*sprTextBack->Render(
			g_ScreenSizeX - 260 - 2, 
			34 );*/
		fnt->printfb( 
			g_ScreenSizeX - 260 - 2, 
			34,
			128,
			32,
			HGETEXT_CENTER|HGETEXT_MIDDLE,
			"Extra %02d", 
			m_iExtraTurns );
	}

	if( m_curEditMode == EM_SELECTTILES )
	{
		for( int iY= 0; iY <= 15; ++iY )
		{
			for( int iX= 0; iX <= 15; ++iX )
			{
				int iTile= iX + 16 * iY;
				float fX= 32.0f * iX + 72.0f;
				float fY= 32.0f * iY + 72.0f;
				sprTile[ iTile ]->RenderEx(
					fX,
					fY,
					0.0f,
					1.0f );
			}
		}
	}

	if( m_curEditMode == EM_SELECTOBJECT )
	{
		for( int iY= 0; iY <= 2; ++iY )
		{
			for( int iX= 0; iX <= 4; ++iX )
			{
				int iObj= iX + 5 * iY;
				if( m_EditObj.size() <= iObj )
				{
					continue;
				}
				if( m_EditObj[ iObj ].m_iIcon != -1 )
				{
					float fX= 32.0f * iX + 72.0f;
					float fY= 32.0f * iY + 72.0f;
					sprObj[ m_EditObj[ iObj ].m_iIcon ]->RenderEx(
						fX,
						fY,
						0.0f,
						1.0f );
				}
			}
		}
	}
}

void Game::SetTile( int iX, int iY, int iTile )
{
	if( iX < 0 || iY < 0 )
	{
		return;
	}
	if( g_EditorEnabled )
	{
		while( m_TileMap.size() <= iY )
		{
			m_TileMap.push_back( TileLine() );
		}
		while( m_TileMap[ iY ].m_TileType.size() <= iX )
		{
			m_TileMap[ iY ].m_TileType.push_back( 0 );
		}
		m_TileMap[ iY ].m_TileType[ iX ]= iTile;

		m_iLevelSizeX= max( iX, m_iLevelSizeX );
		m_iLevelSizeY= max( iY, m_iLevelSizeY );
	}
}

void Game::EditMoveLevel( int dX, int dY )
{
	for( size_t i= 0; i < m_Objects.size(); ++i )
	{
		GameObject& o= m_Objects[ i ];
		o.m_iPosX+= dX;
		o.m_iPosY+= dY;
	}

	std::map< unsigned int, int > MapCopy;
	for( size_t i= 0; i < m_TileMap.size(); ++i )
	{
		TileLine& l= m_TileMap[ i ];
		for( size_t j= 0; j < l.m_TileType.size(); ++j )
		{
			int iTile= l.m_TileType[ j ];
			unsigned int Offset= i << 16 | j;
			MapCopy[ Offset ]= iTile;
		}
	}

	m_TileMap.clear();

	for(	std::map< unsigned int, int >::iterator it= MapCopy.begin();
			it != MapCopy.end();
			++it )
	{
		int iX= it->first & 0xffff;
		int iY= ( it->first >> 16 ) & 0xffff;
		SetTile( iX + dX, iY + dY, it->second );
	}

}

bool Game::LoadFromHTTP( std::string& strURL )
{
	m_strReceivedData= "";
	curl_global_init(CURL_GLOBAL_ALL);

	/* init the curl session */ 
	CURL *curl;
	curl = curl_easy_init();

	int Error= 1;
	if( curl )
	{
		Error= 0;

		/* set URL to get */ 
		Error|= curl_easy_setopt(curl, CURLOPT_URL, strURL.c_str() );

		/* send all data to this function  */ 
		Error|= curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WebData);

		/* get it! */ 
		Error|= curl_easy_perform(curl);

		/* cleanup curl stuff */ 
		curl_easy_cleanup(curl);
	}

	return Error == 0;
}

bool Game::TestNet()
{
	LoadFromHTTP( std::string( "http://www.games-net.de/hosted/tggc/scores/sl/hello.php" ) );
	return m_strReceivedData == "hello";
}

bool Game::FindUser( std::string& strNick )
{
	CURL *curl;
	CURLcode res;

	curl = curl_easy_init();
	if(curl)
	{
		std::stringstream ss;
		ss << "http://www.games-net.de/hosted/tggc/scores/sl/user.php?nick=";
		char* pcName= curl_easy_escape( curl, strNick.c_str(), 0 );
		ss << pcName;
		curl_free( pcName );
		/*ss << "&score=";
		ss << m_iScore;*/
		curl_easy_cleanup(curl);

		std::string strURL= ss.str();
		LoadFromHTTP( strURL );

		return m_strReceivedData == "ok";
	}

	return false;
}

bool Game::CreateUser( std::string& strNick, std::string& strPass )
{
	CURL *curl;
	CURLcode res;

	curl = curl_easy_init();
	if(curl)
	{
		char* pcString;

		std::stringstream ss;
		ss << "http://www.games-net.de/hosted/tggc/scores/sl/create.php?nick=";
		pcString= curl_easy_escape( curl, strNick.c_str(), 0 );
		ss << pcString;
		curl_free( pcString );
		ss << "&pass=";
		pcString= curl_easy_escape( curl, strPass.c_str(), 0 );
		ss << pcString;
		curl_free( pcString );

		curl_easy_cleanup(curl);

		std::string strURL= ss.str();
		LoadFromHTTP( strURL );

		return m_strReceivedData == "ok";
	}

	return false;
}

bool Game::Login( std::string& strNick, std::string& strPass )
{
	CURL *curl;
	CURLcode res;

	curl = curl_easy_init();
	if(curl)
	{
		char* pcString;

		std::stringstream ss;
		ss << "http://www.games-net.de/hosted/tggc/scores/sl/login.php?nick=";
		pcString= curl_easy_escape( curl, strNick.c_str(), 0 );
		ss << pcString;
		curl_free( pcString );
		ss << "&pass=";
		pcString= curl_easy_escape( curl, strPass.c_str(), 0 );
		ss << pcString;
		curl_free( pcString );

		curl_easy_cleanup(curl);

		std::string strURL= ss.str();
		LoadFromHTTP( strURL );

		return m_strReceivedData == "ok";
	}

	return false;
}

bool Game::GetLevelData( std::string& strNick, std::string& strPass, int iLevel )
{
	CURL *curl;
	CURLcode res;

	curl = curl_easy_init();
	if(curl)
	{
		char* pcString;

		std::stringstream ss;
		ss << "http://www.games-net.de/hosted/tggc/scores/sl/level.php?nick=";
		pcString= curl_easy_escape( curl, strNick.c_str(), 0 );
		ss << pcString;
		curl_free( pcString );
		ss << "&pass=";
		pcString= curl_easy_escape( curl, strPass.c_str(), 0 );
		ss << pcString;
		curl_free( pcString );
		if( iLevel >= 0 )
		{
			ss << "&level=";
			ss << iLevel;
		}

		curl_easy_cleanup(curl);

		std::string strURL= ss.str();
		return LoadFromHTTP( strURL );
	}

	return false;
}

bool Game::Solve( std::string& strNick, std::string& strPass, int iLevel )
{
	CURL *curl;
	CURLcode res;

	curl = curl_easy_init();
	if(curl)
	{
		char* pcString;

		std::stringstream ss;
		ss << "http://www.games-net.de/hosted/tggc/scores/sl/solved.php?nick=";
		pcString= curl_easy_escape( curl, strNick.c_str(), 0 );
		ss << pcString;
		curl_free( pcString );
		ss << "&pass=";
		pcString= curl_easy_escape( curl, strPass.c_str(), 0 );
		ss << pcString;
		curl_free( pcString );
		ss << "&level=";
		ss << iLevel;
		if( g_bNoTwitter )
		{
			ss << "&notwit=1";
		}
		curl_easy_cleanup(curl);

		std::string strURL= ss.str();
		LoadFromHTTP( strURL );

		return m_strReceivedData == "ok";
	}

	return false;
}

bool Game::Sleep( std::string& strNick, std::string& strPass )
{
	CURL *curl;
	CURLcode res;

	curl = curl_easy_init();
	if(curl)
	{
		char* pcString;

		std::stringstream ss;
		ss << "http://www.games-net.de/hosted/tggc/scores/sl/sleep.php?nick=";
		pcString= curl_easy_escape( curl, strNick.c_str(), 0 );
		ss << pcString;
		curl_free( pcString );
		ss << "&pass=";
		pcString= curl_easy_escape( curl, strPass.c_str(), 0 );
		ss << pcString;
		curl_free( pcString );
		ss << "&level=";
		ss << m_Level;
		ss << "&posxy=";
		int iXY= m_Objects[ m_PlayerID ].m_iPosX + ( m_Objects[ m_PlayerID ].m_iPosY << 16 );
		ss << iXY;
		ss << "&turns=";
		ss << m_iTurns;
		ss << "&msg=";
		pcString= curl_easy_escape( curl, m_strTextEdit.c_str(), 0 );
		ss << pcString;
		curl_free( pcString );
		if( g_bNoTwitter )
		{
			ss << "&notwit=1";
		}

		curl_easy_cleanup(curl);

		std::string strURL= ss.str();
		LoadFromHTTP( strURL );

		return m_strReceivedData == "ok";
	}

	return false;
}

size_t Game::WebData(void *ptr, size_t size, size_t nmemb, void *stream)
{
	int Data = size * nmemb;
	if( m_strReceivedData.size() < 100000 )
	{
		for( int i= 0; i < Data; ++i )
		{
			m_strReceivedData+= ((char*)ptr)[ i ];
		}
	}
	return Data;
}

void Game::SelectLevel()
{
	m_strPrompt= "Choose a level:";
	m_strTextEdit= "";
	m_CurTextEdit= TEM_LEVEL;

	//m_iMenuMode= MENU_PREGAME;
}

Game::EditObjConfig::EditObjConfig( int iIcon, int iType, int iSubType )
{
	m_iIcon= iIcon;
	m_iType= iType;
	m_iSubType= iSubType;
}

//	EOF: Game.cpp
