/*!
	\file Game.h

	\brief Header Datei der Klasse Game
	\author Christian Roesch
	\date 21.6.2010

	Erklaerung
*/

#ifndef Game_H
#define Game_H

#include <vector>
#include <map>

class GameObject;
class Move;

/*!
	\brief Kurzgesagt

	Tut _nichts_.
*/

class Game
{
public:
	Game();

	~Game();

	void ResetGame();
	void PrepareGame();
	void PrepareGameWorld( bool ResetPlayer );

	void StartGame();

	bool FrameFunc();

	void FrameCalculation( float dt );

	void KeyInput();

	void MouseInput( float dt );

	void OnMouseClick( int iMenu );
	void OnTextEnter( bool bOk );

	void SleepPlayTest( int UserDir );

	bool RenderFunc();

	void DrawEditor();

	void DrawUI();

	void DrawObjects();

	void DrawTiles();

	int GetLevel();

	int GetTile( int iX, int iY );
	void SetTile( int iX, int iY, int iTile );
	
	bool IsWalkable( int iX, int iY );
	bool IsSliding( int iX, int iY );
	int IsChangingDir( int iX, int iY );
	int IsWin( int iX, int iY );

	void TileToScreen( float& fX, float& fY );
	void ScreenToTile( float& fX, float& fY );

	void LoadLevel( std::vector <std::string>& LevelScript, bool bResetPlayer );
	void SaveEditorLevel();

	void RemovePlayer();

	void EditMoveLevel( int dX, int dY );
	
	void WinLevel();
	void SleepLevel();
	void SelectLevel();

private:
	void NextLevel();
	void ExecuteMove();
	bool MoveAllowed( Move* pMove );
	
	void CheckAction( int ObjectID );

	bool IsTurnActive();

	int m_SelectionID;
	int m_HoverID;

	enum GameState
	{
		GS_ENTERNICK,
		GS_ENTERPASS,
		GS_CREATEACCOUNT,
		GS_NORMAL,
		GS_COULDNOTSOLVE,
		GS_COULDNOTSLEEP,
		GS_COULDNOTLOAD,
	};
	GameState m_GameState;

	std::string m_strNickName;
	std::string m_strPassword;

	void AddPlayerObject( int iStartPosOffset );
	int m_PlayerID;
	std::vector< GameObject > m_Objects;

	void AddAsciiLevelLine( std::string& strAct );
	void AddBinLevelLine( std::string& strAct );
	struct TileLine
	{
		std::vector<int> m_TileType;
	};
	std::vector< TileLine > m_TileMap;
	static const int c_BackgroundTile= 0;

	int m_Level;
	bool m_bReplay;
	int m_iTurns;
	int m_iExtraTurns;

	int m_iLevelSizeX;
	int m_iLevelSizeY;

	float m_fGameTime;

	Move* m_pCurMove;

	std::string m_strMessage;

	enum TextEditMode
	{
		TEM_DISABLED,
		TEM_USER,
		TEM_PASS,
		TEM_SLEEPMSG,
		TEM_LEVEL,
		TEM_EDITOROBJTEXT,
	};
	TextEditMode m_CurTextEdit;
	std::string m_strPrompt;
	std::string m_strTextEdit;

	bool IsIngameActive();
	int OnMenu();
	enum Menus
	{
		MENU_PREGAME,
		MENU_INGAME,
		MENU_NEXTLEVEL,
		MENU_RETRY,
		MENU_SLEEP,
		MENU_EDITOR,
		MENU_OBJEDIT,
		MENU_TURNS,
		MENU_PLAYTEST,
		MENU_COUNT
	};
	std::vector< std::string > m_strMenu[ MENU_COUNT ];
	Menus m_iMenuMode;

	bool m_bWantsToQuit;

	void PlayTest();
	void SelectTile();
	void SelectObject();
	void EditObject();
	void EditTurns();
	std::string GetEditFileName();

	std::map< int, GameObject > m_ServerEmulator;
	enum EditMode
	{
		EM_TILES,
		EM_SELECTTILES,
		EM_OBJECTS,
		EM_SELECTOBJECT,
		EM_OBJECTEDIT,
		EM_TURNS,
		EM_PLAYTEST,
	};
	EditMode m_curEditMode;
	int m_PlayTestUser;
	int m_EditTile;
	struct EditObjConfig
	{
		EditObjConfig( 
			int iIcon,
			int iType,
			int iSubType );

		int m_iIcon;
		int m_iType;
		int m_iSubType;
	};
	std::vector< EditObjConfig > m_EditObj;
	int m_EditObjID;

	//Webzeug...
	static std::string m_strReceivedData;
	static size_t WebData(void *ptr, size_t size, size_t nmemb, void *stream);
	bool LoadFromHTTP( std::string& strURL );
public:
	bool TestNet();
private:
	bool FindUser( std::string& strNick );
	bool CreateUser( std::string& strNick, std::string& strPass );
	bool Login( std::string& strNick, std::string& strPass );
	bool GetLevelData( std::string& strNick, std::string& strPass, int Level );
	bool Solve( std::string& strNick, std::string& strPass, int iLevel );
	bool Sleep( std::string& strNick, std::string& strPass );
};


#endif

class Game;


//	EOF: Game.h
