/*!
	\file GameObject.cpp

	\brief cpp-Datei der Klasse CGameObject
	\author Christian Roesch
	\date 21.6.2010

	Erklaerung	
*/

#include "GameObject.h"
#include "globals.h"
#include "helper.h"

#include <cmath>
#include "resources.h"
#include "Game.h"
#include "Move.h"

/*!
	Der Konstruktor
*/
GameObject::GameObject()
{
	Reset();
}
void GameObject::Reset()
{
	m_Type= GHOSTSTART;
	m_iSubType= 0;
	m_iPosX= 0;
	m_iPosY= 0;
	m_strText= "";
	m_pMove= NULL;
}

void GameObject::draw()
{
	if( IsDisabled() )
	{
		return;
	}

	ObjectTextures ot= OT_PLAYER;
	switch( m_Type )
	{
	case OIL:
		if( m_iSubType != -1 )
		{
			ot= OT_COOP;
		}
		break;
	case MOVER:
		ot= (ObjectTextures)( OT_MOVER + ( m_iSubType + 1 ) );
		break;
	case TALKER:
		ot= OT_TALKER;
		break;
	case GHOSTSTART:
		ot= OT_START;
		break;
	}
	float fPosX= (float)m_iPosX;
	float fPosY= (float)m_iPosY;
	if( m_pMove != NULL )
	{
		int iTargetX;
		int iTargetY;
		m_pMove->GetTarget( iTargetX, iTargetY );
		fPosX= fPosX * m_pMove->m_fTimeLeft + (float)iTargetX * (1.0f - m_pMove->m_fTimeLeft );
		fPosY= fPosY * m_pMove->m_fTimeLeft + (float)iTargetY * (1.0f - m_pMove->m_fTimeLeft );
	}
	g_Game.TileToScreen( fPosX, fPosY );
	sprObj[ ot ]->RenderEx( 
		fPosX, 
		fPosY,
		0.0f,
		g_fZoom);
}

bool GameObject::IsSelectable()
{
	if( IsDisabled() )
	{
		return false;
	}
	//return m_Type != GHOSTSTART;
	return true;
}

bool GameObject::IsStart()
{
	return m_Type == GHOSTSTART;
}

bool GameObject::AcceptsMove( Move* pMove )
{
	if( m_pMove != NULL )
	{
		return false;
	}

	if( !IsSelectable() )
	{
		return false;
	}

	if( IsDisabled() )
	{
		return false;
	}

	switch( m_Type )
	{
	case MOVER:
		if( m_iSubType >= 0 )
		{
			if( pMove->m_iDir != m_iSubType )
			{
				return false;
			}
		}
		break;
	case OIL:
		if( m_iSubType != -1 )
		{
			return false;
		}
		break;
	case TALKER:
	case GHOSTSTART:
		return false;
		break;
	}

	m_pMove= pMove;
	return true;
}

bool GameObject::IsBlocking()
{
	if( IsDisabled() )
	{
		return false;
	}

	switch( m_Type )
	{
	case GHOSTSTART:
		return false;
		break;
	}

	return true;
}

bool GameObject::IsPlayer()
{
	return ( m_Type == OIL && m_iSubType == -1 );
}

bool GameObject::IsOil()
{
	return m_Type == OIL;
}

bool GameObject::IsDisabled()
{
	return m_Type == DISABLED;
}

void GameObject::TryToDisable()
{
	if( IsPlayer() )
	{
		return;
	}
	if( m_Type == MOVER )
	{
		return;
	}

	m_Type= DISABLED;
}

//	EOF: GameObject.cpp
