/*
** Haaf's Game Engine 1.8
** Copyright (C) 2003-2007, Relish Games
** hge.relishgames.com
**
** hge_tut03 - Using helper classes
*/


#include <cmath>
#include <ctime>
#include <vector>

#include "hge.h"
#include "hgesprite.h"
#include "hgefont.h"

#include "helper.h"
#include "globals.h"

#include "game.h"
#include "resources.h"

#include "setup/fwzSetup.h"

#include <shlobj.h>
#include <fstream>
#include "TextFileReader.h"

int ParseCmdLine( const char* pcCommand )
{	
	int iCmdLineFound= 0;
	{
		int i= 0;
		int iNumber= 0;
		int iBool= -1;
		{
			while( pcCommand[ i ] != 0 )
			{
				if(  pcCommand[ i ] == '+' )
				{
					iBool= 1;
					iNumber= 0;
				}
				else if(  pcCommand[ i ] == '-' )
				{
					iBool= 0;
					iNumber= 0;
				}
				else if( pcCommand[ i ] >= '0' && pcCommand[ i ] <= '9' )
				{
					iNumber= 10 * iNumber + pcCommand[ i  ] - '0';
				}
				else if(  pcCommand[ i ] == 'w' )
				{
					if( iBool != -1 )
					{
						g_bWindow= iBool == 1;
					}
					iBool= -1;
					iNumber= 0;
				}
				else if(  pcCommand[ i ] == 't' )
				{
					if( iBool != -1 )
					{
						g_bNoTwitter= iBool != 1;
					}
					iBool= -1;
					iNumber= 0;
				}				
				/*else if(  pcCommand[ i ] == 'a' )
				{
					if( iBool != -1 )
					{
						//bMultisampling= iBool == 1;
					}
					iBool= -1;
					iNumber= 0;
				}*/
				else if(  pcCommand[ i ] == 'x' )
				{
					if( iNumber > 0 )
					{
						g_ScreenSizeX= iNumber;
						iCmdLineFound++;
					}
					iNumber= 0;
				}
				else if(  pcCommand[ i ] == 'y' )
				{
					if( iNumber > 0 )
					{
						g_ScreenSizeY= iNumber;
						iCmdLineFound++;
					}
					iNumber= 0;
				}
				else
				{
					iNumber= 0;
					iBool= -1;
				}

				i++;
			}
		}
	}

	return iCmdLineFound;
}

int Setup( HINSTANCE hInstance )
{
	g_ScreenSizeX= GetSystemMetrics( SM_CXSCREEN );
	g_ScreenSizeY= GetSystemMetrics( SM_CYSCREEN );
	g_bWindow= false;
	g_bNoTwitter= false;
	
	char path[MAX_PATH];
	SHGetFolderPath( 0, CSIDL_PERSONAL, 0, SHGFP_TYPE_CURRENT, path );

	std::string strParamFile( path );
	strParamFile+= "\\slippery.param";

	FrameWork::TextFileReader tfr;
	if( tfr.read( strParamFile.c_str() ) )
	{
		if( tfr.getLineCount() > 0 )
		{
			ParseCmdLine( tfr.getFileLines()[ 0 ].c_str() );
		}
	}

	char* pcCommand= GetCommandLine();
	int iCmdLineFound= ParseCmdLine( pcCommand );

	fwzSettings setup;
	setup.hInstance = hInstance;

	// These are just some default settings I found convenient to work with.
	// If you modify them, the setupbox will initialize accordingly.
	setup.scrBPP = 32;
	setup.nVsync = 1;
	setup.nMultisample = 1; // still buggy

	if( iCmdLineFound < 2 )
	{
#	ifdef BLUBB
		// It's a fairly good idea not to set always-on-top when
		// you're debugging, since otherwise it gets stuck
		// on top your debug window.
		setup.nAlwaysOnTop = 0;
		setup.scrWidth = 1024;
		setup.scrHeight = 768;
		setup.nWindowed = 1;
#	else
		setup.nAlwaysOnTop = g_bNoTwitter ? 0 : 1;
		setup.scrWidth = g_ScreenSizeX;
		setup.scrHeight = g_ScreenSizeY;
		setup.nWindowed = g_bWindow ? 1 : 0;
		if (!OpenSetupDialog(&setup))
		{
			return -1;
		}
		else
		{
			//bMultisampling= setup.nMultisample != 0;

			g_ScreenSizeX= setup.scrWidth;
			g_ScreenSizeY= setup.scrHeight;
			g_bWindow= setup.nWindowed != 0;
			g_bNoTwitter= setup.nAlwaysOnTop != 0;
		}
#	endif
	}

	std::ofstream ofs ( strParamFile.c_str() );
	ofs << g_ScreenSizeX << "x ";
	ofs << g_ScreenSizeY << "y ";
	ofs << (g_bWindow?"+":"-") << "w ";
	ofs << (g_bNoTwitter?"-":"+") << "t ";
	ofs.close();

	return 0;
}

bool FrameFunc()
{
	return g_Game.FrameFunc();
}

bool RenderFunc()
{
	return g_Game.RenderFunc();
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, int)
{
	if( Setup( hInstance ) < 0 )
	{
		return 0;
	}

	//firewall workaraound?
	if( !g_EditorEnabled )
	{
		g_Game.TestNet();
	}

	g_hge = hgeCreate(HGE_VERSION);

	g_hge->Random_Seed( (int)time( NULL ));

	g_hge->System_SetState(HGE_LOGFILE, "error.log");
	g_hge->System_SetState(HGE_FRAMEFUNC, FrameFunc);
	g_hge->System_SetState(HGE_RENDERFUNC, RenderFunc);
	g_hge->System_SetState(HGE_TITLE, "Slippery");
	g_hge->System_SetState(HGE_FPS, 100);
	g_hge->System_SetState(HGE_WINDOWED, g_bWindow);
	g_hge->System_SetState(HGE_SCREENWIDTH, g_ScreenSizeX );
	g_hge->System_SetState(HGE_SCREENHEIGHT, g_ScreenSizeY );
	g_hge->System_SetState(HGE_SCREENBPP, 32);
	g_hge->System_SetState(HGE_SHOWSPLASH, false );

	if(g_hge->System_Initiate()) 
	{
		LoadResource();

		g_Game.PrepareGame();

		// Let's rock now!
		//g_hge->Music_Play(musGame, true);
		g_hge->System_Start();

		// Delete created objects and free loaded resources
		FreeResource();
	}

	// Clean up and shutdown
	g_hge->System_Shutdown();
	g_hge->Release();
	return 0;
}
